@@ -1,7 +1,7 @@
 // vim: set tabstop=4 shiftwidth=4 noexpandtab:
 /*
 Gwenview: an image viewer
-Copyright 2007 Aurélien Gâteau <aurelien.gateau@free.fr>
+Copyright 2007 Aurélien Gâteau <agateau@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -40,7 +40,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 namespace Gwenview {
 
 
-enum GroupRow { NoGroup = -1, GeneralGroup, ExifGroup, IptcGroup };
+enum GroupRow {
+	NoGroupSpace= -2,
+	NoGroup = -1,
+	GeneralGroup,
+	ExifGroup,
+	IptcGroup,
+	XmpGroup
+};
 
 
 class MetaInfoGroup {
@@ -51,8 +58,8 @@ public:
 
 	class Entry {
 	public:
-		Entry(const QString& key, const QString& label)
-		: mKey(key), mLabel(label.trimmed())
+		Entry(const QString& key, const QString& label, const QString& value)
+		: mKey(key), mLabel(label.trimmed()), mValue(value.trimmed())
 		{}
 
 		QString key() const { return mKey; }
@@ -91,18 +98,13 @@ public:
 
 
 	void addEntry(const QString& key, const QString& label, const QString& value) {
-		// key aren't always unique (for example, "Iptc.Application2.Keywords"
-		// may appear multiple times). In this case, append the value at the
-		// end of the existing one.
-		Entry* entry = getEntryForKey(key);
-		if (entry) {
-			entry->appendValue(value);
-		} else {
-			entry = new Entry(key, label);
-			entry->setValue(value);
-			mList << entry;
-			mRowForKey[key] = mList.size() - 1;
-		}
+		addEntry(new Entry(key, label, value));
+	}
+
+
+	void addEntry(Entry* entry) {
+		mList << entry;
+		mRowForKey[entry->key()] = mList.size() - 1;
 	}
 
 
@@ -186,15 +188,6 @@ struct ImageMetaInfoModelPrivate {
 	}
 
 
-	void notifyGroupFilled(MetaInfoGroup* group, const QModelIndex& parent) {
-		if (group->size() == 0) {
-			return;
-		}
-		mModel->beginInsertRows(parent, 0, group->size() - 1);
-		mModel->endInsertRows();
-	}
-
-
 	void setGroupEntryValue(GroupRow groupRow, const QString& key, const QString& value) {
 		MetaInfoGroup* group = mMetaInfoGroupVector[groupRow];
 		int entryRow = group->getRowForKey(key);
@@ -211,13 +204,17 @@ struct ImageMetaInfoModelPrivate {
 
 	QVariant displayData(const QModelIndex& index) const {
 		if (index.internalId() == NoGroup) {
-			if (index.column() > 0) {
+			if (index.column() != 0) {
 				return QVariant();
 			}
 			QString label = mMetaInfoGroupVector[index.row()]->label();
 			return QVariant(label);
 		}
 
+		if (index.internalId() == NoGroupSpace) {
+			return QVariant(QString());
+		}
+
 		MetaInfoGroup* group = mMetaInfoGroupVector[index.internalId()];
 		if (index.column() == 0) {
 			return group->getLabelForKeyAt(index.row());
@@ -234,16 +231,57 @@ struct ImageMetaInfoModelPrivate {
 		group->addEntry("General.Time", i18nc("@item:intable", "File Time"), QString());
 		group->addEntry("General.ImageSize", i18nc("@item:intable", "Image Size"), QString());
 	}
+
+
+	template <class Container, class Iterator>
+	void fillExivGroup(const QModelIndex& parent, MetaInfoGroup* group, const Container& container) {
+		// key aren't always unique (for example, "Iptc.Application2.Keywords"
+		// may appear multiple times) so we can't know how many rows we will
+		// insert before going through them. That's why we create a hash
+		// before.
+		typedef QHash<QString, MetaInfoGroup::Entry*> EntryHash;
+		EntryHash hash;
+
+		Iterator
+			it = container.begin(),
+			end = container.end();
+
+		if (it == end) {
+			return;
+		}
+
+		for (;it != end; ++it) {
+			QString key = QString::fromUtf8(it->key().c_str());
+			QString label = QString::fromLocal8Bit(it->tagLabel().c_str());
+			std::ostringstream stream;
+			stream << *it;
+			QString value = QString::fromLocal8Bit(stream.str().c_str());
+
+			EntryHash::iterator hashIt = hash.find(key);
+			if (hashIt != hash.end()) {
+				hashIt.value()->appendValue(value);
+			} else {
+				hash.insert(key, new MetaInfoGroup::Entry(key, label, value));
+			}
+		}
+
+		mModel->beginInsertRows(parent, 0, hash.size() - 1);
+		Q_FOREACH(MetaInfoGroup::Entry* entry, hash) {
+			group->addEntry(entry);
+		}
+		mModel->endInsertRows();
+	}
 };
 
 
 ImageMetaInfoModel::ImageMetaInfoModel()
 : d(new ImageMetaInfoModelPrivate) {
 	d->mModel = this;
-	d->mMetaInfoGroupVector.resize(3);
+	d->mMetaInfoGroupVector.resize(4);
 	d->mMetaInfoGroupVector[GeneralGroup] = new MetaInfoGroup(i18nc("@title:group General info about the image", "General"));
-	d->mMetaInfoGroupVector[ExifGroup] = new MetaInfoGroup(i18nc("@title:group", "Exif"));
-	d->mMetaInfoGroupVector[IptcGroup] = new MetaInfoGroup(i18nc("@title:group", "Iptc"));
+	d->mMetaInfoGroupVector[ExifGroup] = new MetaInfoGroup("EXIF");
+	d->mMetaInfoGroupVector[IptcGroup] = new MetaInfoGroup("IPTC");
+	d->mMetaInfoGroupVector[XmpGroup]  = new MetaInfoGroup("XMP");
 	d->initGeneralGroup();
 }
 
@@ -285,66 +323,48 @@ void ImageMetaInfoModel::setImageSize(const QSize& size) {
 }
 
 
-template <class iterator>
-static void fillExivGroup(MetaInfoGroup* group, iterator begin, iterator end) {
-	iterator it = begin;
-	for (;it != end; ++it) {
-		QString key = QString::fromUtf8(it->key().c_str());
-		QString label = QString::fromLocal8Bit(it->tagLabel().c_str());
-		std::ostringstream stream;
-		stream << *it;
-		QString value = QString::fromLocal8Bit(stream.str().c_str());
-		group->addEntry(key, label, value);
-	}
-}
-
-
 void ImageMetaInfoModel::setExiv2Image(const Exiv2::Image* image) {
 	MetaInfoGroup* exifGroup = d->mMetaInfoGroupVector[ExifGroup];
 	MetaInfoGroup* iptcGroup = d->mMetaInfoGroupVector[IptcGroup];
+	MetaInfoGroup* xmpGroup  = d->mMetaInfoGroupVector[XmpGroup];
 	QModelIndex exifIndex = index(ExifGroup, 0);
 	QModelIndex iptcIndex = index(IptcGroup, 0);
+	QModelIndex xmpIndex  = index(XmpGroup, 0);
 	d->clearGroup(exifGroup, exifIndex);
 	d->clearGroup(iptcGroup, iptcIndex);
+	d->clearGroup(xmpGroup,  xmpIndex);
 
 	if (!image) {
 		return;
 	}
 
-	if (image->supportsMetadata(Exiv2::mdExif)) {
+	if (image->checkMode(Exiv2::mdExif) & Exiv2::amRead) {
 		const Exiv2::ExifData& exifData = image->exifData();
-
-		fillExivGroup(
-			exifGroup,
-			exifData.begin(),
-			exifData.end()
-			);
-
-		d->notifyGroupFilled(exifGroup, exifIndex);
+		d->fillExivGroup<Exiv2::ExifData, Exiv2::ExifData::const_iterator>(exifIndex, exifGroup, exifData);
 	}
 
-	if (image->supportsMetadata(Exiv2::mdIptc)) {
+	if (image->checkMode(Exiv2::mdIptc) & Exiv2::amRead) {
 		const Exiv2::IptcData& iptcData = image->iptcData();
+		d->fillExivGroup<Exiv2::IptcData, Exiv2::IptcData::const_iterator>(iptcIndex, iptcGroup, iptcData);
+	}
 
-		fillExivGroup(
-			iptcGroup,
-			iptcData.begin(),
-			iptcData.end()
-			);
-
-		d->notifyGroupFilled(iptcGroup, iptcIndex);
+	if (image->checkMode(Exiv2::mdXmp) & Exiv2::amRead) {
+		const Exiv2::XmpData& xmpData = image->xmpData();
+		d->fillExivGroup<Exiv2::XmpData, Exiv2::XmpData::const_iterator>(xmpIndex, xmpGroup, xmpData);
 	}
 }
 
 
 void ImageMetaInfoModel::getInfoForKey(const QString& key, QString* label, QString* value) const {
 	MetaInfoGroup* group;
-	if (key.startsWith("General")) {
+	if (key.startsWith(QLatin1String("General"))) {
 		group = d->mMetaInfoGroupVector[GeneralGroup];
-	} else if (key.startsWith("Exif")) {
+	} else if (key.startsWith(QLatin1String("Exif"))) {
 		group = d->mMetaInfoGroupVector[ExifGroup];
-	} else if (key.startsWith("Iptc")) {
+	} else if (key.startsWith(QLatin1String("Iptc"))) {
 		group = d->mMetaInfoGroupVector[IptcGroup];
+	} else if (key.startsWith(QLatin1String("Xmp"))) {
+		group = d->mMetaInfoGroupVector[XmpGroup];
 	} else {
 		kWarning() << "Unknown metainfo key" << key;
 		return;
@@ -370,22 +390,19 @@ QString ImageMetaInfoModel::keyForIndex(const QModelIndex& index) const {
 
 
 QModelIndex ImageMetaInfoModel::index(int row, int col, const QModelIndex& parent) const {
+	if (col < 0 || col > 1) {
+		return QModelIndex();
+	}
 	if (!parent.isValid()) {
 		// This is a group
-		if (col > 0) {
+		if (row < 0 || row >= d->mMetaInfoGroupVector.size()) {
 			return QModelIndex();
 		}
-		if (row >= d->mMetaInfoGroupVector.size()) {
-			return QModelIndex();
-		}
-		return createIndex(row, col, NoGroup);
+		return createIndex(row, col, col == 0 ? NoGroup : NoGroupSpace);
 	} else {
 		// This is an entry
-		if (col > 1) {
-			return QModelIndex();
-		}
 		int group = parent.row();
-		if (row >= d->mMetaInfoGroupVector[group]->size()) {
+		if (row < 0 || row >= d->mMetaInfoGroupVector[group]->size()) {
 			return QModelIndex();
 		}
 		return createIndex(row, col, group);
@@ -397,7 +414,7 @@ QModelIndex ImageMetaInfoModel::parent(const QModelIndex& index) const {
 	if (!index.isValid()) {
 		return QModelIndex();
 	}
-	if (index.internalId() == NoGroup) {
+	if (index.internalId() == NoGroup || index.internalId() == NoGroupSpace) {
 		return QModelIndex();
 	} else {
 		return createIndex(index.internalId(), 0, NoGroup);
