@@ -50,6 +50,12 @@ public:
 	PreviewItemDelegate(ThumbnailView*);
 	~PreviewItemDelegate();
 
+	enum ContextBarMode {
+		NoContextBar,            /** Do not show context bar at all */
+		SelectionOnlyContextBar, /** Only show the +/- button */
+		FullContextBar           /** Show all buttons, provided there is enough room */
+	};
+
 	enum ThumbnailDetail {
 		FileNameDetail = 1,
 		DateDetail     = 2,
@@ -66,6 +72,14 @@ public:
 
 	void setThumbnailDetails(ThumbnailDetails);
 
+	ContextBarMode contextBarMode() const;
+
+	void setContextBarMode(ContextBarMode);
+
+	Qt::TextElideMode textElideMode() const;
+
+	void setTextElideMode(Qt::TextElideMode);
+
 	virtual void paint( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const;
 	virtual QSize sizeHint( const QStyleOptionViewItem & /*option*/, const QModelIndex & /*index*/ ) const;
 
