@@ -75,6 +75,7 @@ KLineal::KLineal( QWidget *parent )
   : QWidget( parent ),
     mDragging( false ),
     mShortEdgeLen( 70 ),
+    mCloseAction( 0 ),
     mLenMenu( 0 ),                // INFO This member could be eventually deleted
                                   // since if mFullScreenAction is initialized
                                   // mLenMenu should have been, too.
@@ -248,21 +249,28 @@ KLineal::KLineal( QWidget *parent )
 
 KLineal::~KLineal()
 {
-    delete mTrayIcon;
+  delete mTrayIcon;
 }
 
 void KLineal::createSystemTray()
 {
-    KAction *closeAction = KStandardAction::close( this, SLOT( slotClose() ), this );
-    mActionCollection->addAction( "close", closeAction );
-    mMenu->addAction( closeAction );
+  if ( !mCloseAction ) {
+    mCloseAction = KStandardAction::close( this, SLOT( slotClose() ), this );
+    mActionCollection->addAction( "close", mCloseAction );
+    mMenu->addAction( mCloseAction );
 
     mCloseButton = new QToolButton( this );
-    mCloseButton->setIcon( closeAction->icon() );
-    mCloseButton->setToolTip( closeAction->text().remove( '&' ) );
+    mCloseButton->setIcon( mCloseAction->icon() );
+    mCloseButton->setToolTip( mCloseAction->text().remove( '&' ) );
     connect( mCloseButton, SIGNAL( clicked() ), this, SLOT( slotClose() ) );
-    mTrayIcon = new KRulerSystemTray( KIcon( "kruler" ), this,mActionCollection );
-    mTrayIcon->show();
+  } else {
+    mCloseAction->setVisible( true );
+  }
+  
+  if ( !mTrayIcon ) {
+    mTrayIcon = new KRulerSystemTray( "kruler", this, mActionCollection );
+    mTrayIcon->setCategory( KStatusNotifierItem::ApplicationStatus );
+  }
 }
 
 
@@ -629,32 +637,34 @@ void KLineal::slotPreferences()
   advancedConfig.setupUi( advancedConfigWidget );
   dialog->addPage( advancedConfigWidget, i18n( "Advanced" ), "preferences-other" );
 
-  connect(dialog, SIGNAL(settingsChanged(const QString&)), SLOT(loadConfig()));
+  connect( dialog, SIGNAL( settingsChanged( const QString& ) ), SLOT( loadConfig() ) );
   dialog->exec();
   delete dialog;
 }
 
 void KLineal::loadConfig()
 {
-    mColor = RulerSettings::self()->bgColor();
-    mScaleFont = RulerSettings::self()->scaleFont();
-    saveSettings();
+  mColor = RulerSettings::self()->bgColor();
+  mScaleFont = RulerSettings::self()->scaleFont();
+  saveSettings();
 
-    if ( RulerSettings::self()->trayIcon() ) {
-        if ( !mTrayIcon ) {
-            createSystemTray();
-            //need to adjust button
-            adjustButtons();
-        }
-        else
-            mTrayIcon->show();
+  if ( RulerSettings::self()->trayIcon() ) {
+    if ( !mTrayIcon ) {
+      createSystemTray();
+      //need to adjust button
+      adjustButtons();
     }
-    else {
-        if ( mTrayIcon ) {
-            mTrayIcon->hide();
-        }
+  } else {
+    if ( mTrayIcon ) {
+      delete mTrayIcon;
+      mTrayIcon = 0;
     }
-    repaint();
+
+    if ( mCloseAction ) {
+      mCloseAction->setVisible( false );
+    }
+  }
+  repaint();
 }
 
 
@@ -876,7 +886,7 @@ void KLineal::mouseMoveEvent( QMouseEvent *inEvent )
   } else {
     QPoint p = QCursor::pos();
 
-    switch (mOrientation) {
+    switch ( mOrientation ) {
     case North:
       p.setY( p.y() - 46 );
       break;
@@ -985,9 +995,25 @@ void KLineal::wheelEvent( QWheelEvent *e )
       saveSettings();
     }
   } else { // changing length
+    int oldLen = mLongEdgeLen;
     int newLength = mLongEdgeLen + numSteps;
     reLengthAbsolute( newLength );
-    mLabel->setText( i18n( "Length: %1 px", newLength ) );
+    mLabel->setText( i18n( "Length: %1 px", mLongEdgeLen ) );
+
+    // when holding shift relength at the other side
+    if ( e->modifiers() & Qt::ShiftModifier ) {
+      int change = mLongEdgeLen - oldLen;
+
+      QPoint dist;
+
+      if ( mOrientation == North || mOrientation == South ) {
+        dist.setX( -change );
+      } else {
+        dist.setY( -change );
+      }
+
+      move( pos() + dist );
+    }
   }
 
   QWidget::wheelEvent( e );
