@@ -98,6 +98,7 @@ void KSnapshotObject::changeUrl( const QString &url )
 }
 
 
+// NOTE: widget == NULL if called from dbus interface
 bool KSnapshotObject::save( const QString &filename, QWidget* widget )
 {
     return save( KUrl( filename ), widget);
@@ -106,6 +107,7 @@ bool KSnapshotObject::save( const QString &filename, QWidget* widget )
 bool KSnapshotObject::save( const KUrl& url, QWidget *widget )
 {
     if ( KIO::NetAccess::exists( url, KIO::NetAccess::DestinationSide, widget ) ) {
+        // NOTE: widget == NULL if called from dbus interface
         const QString title = i18n( "File Exists" );
         const QString text = i18n( "<qt>Do you really want to overwrite <b>%1</b>?</qt>" , url.prettyUrl());
         if (KMessageBox::Continue != KMessageBox::warningContinueCancel( widget, text, title, KGuiItem(i18n("Overwrite")) ) )
@@ -120,7 +122,7 @@ bool KSnapshotObject::saveEqual( const KUrl& url,QWidget *widget )
 {
     QByteArray type = "PNG";
     QString mime = KMimeType::findByUrl( url.fileName(), 0, url.isLocalFile(), true )->name();
-    QStringList types = KImageIO::typeForMime(mime);
+    const QStringList types = KImageIO::typeForMime(mime);
     if ( !types.isEmpty() )
         type = types.first().toLatin1();
 
@@ -144,8 +146,8 @@ bool KSnapshotObject::saveEqual( const KUrl& url,QWidget *widget )
     if ( !ok ) {
         kWarning() << "KSnapshot was unable to save the snapshot" ;
 
-        QString caption = i18n("Unable to Save Image");
-        QString text = i18n("KSnapshot was unable to save the image to\n%1.", url.prettyUrl());
+        const QString caption = i18n("Unable to Save Image");
+        const QString text = i18n("KSnapshot was unable to save the image to\n%1.", url.prettyUrl());
         KMessageBox::error(widget, text, caption);
     }
 
@@ -174,3 +176,4 @@ bool KSnapshotObject::saveImage( QIODevice *device, const QByteArray &format )
     QImage snap = snapshot.toImage();
     return imgWriter.write( snap );
 }
+
