@@ -22,7 +22,6 @@
  *
  * ============================================================ */
 
-#include "kdcraw.h"
 #include "kdcraw.moc"
 
 // Qt includes.
@@ -161,9 +160,9 @@ bool KDcraw::loadHalfPreview(QImage& image, const QString& path)
     kDebug(51002) << "Try to use reduced RAW picture extraction" << endl;
 
     LibRaw raw;
-    raw.imgdata.params.use_auto_wb    = 1; // Use automatic white balance.
-    raw.imgdata.params.use_camera_wb  = 1; // Use camera white balance, if possible.
-    raw.imgdata.params.half_size      = 1; // Half-size color image (3x faster than -q).
+    raw.imgdata.params.use_auto_wb    = 1;         // Use automatic white balance.
+    raw.imgdata.params.use_camera_wb  = 1;         // Use camera white balance, if possible.
+    raw.imgdata.params.half_size      = 1;         // Half-size color image (3x faster than -q).
 
     // NOTE: new magic option introduced by LibRaw 0.7.0 to to make better noise filtration.
     raw.imgdata.params.filtering_mode = LIBRAW_FILTERING_AUTOMATIC;
@@ -250,7 +249,7 @@ bool KDcraw::rawFileIdentify(DcrawInfoContainer& identify, const QString& path)
 
 // ----------------------------------------------------------------------------------
 
-bool KDcraw::extractRAWData(const QString& filePath, QByteArray &rawData, DcrawInfoContainer& identify)
+bool KDcraw::extractRAWData(const QString& filePath, QByteArray& rawData, DcrawInfoContainer& identify)
 {
     QFileInfo fileInfo(filePath);
     QString   rawFilesExt(rawFiles());
@@ -336,14 +335,14 @@ bool KDcraw::decodeHalfRAWImage(const QString& filePath, const RawDecodingSettin
 {
     m_rawDecodingSettings                    = rawDecodingSettings;
     m_rawDecodingSettings.halfSizeColorImage = true;
-    return (loadFromDcraw(filePath, imageData, width, height, rgbmax));
+    return (loadFromLibraw(filePath, imageData, width, height, rgbmax));
 }
 
 bool KDcraw::decodeRAWImage(const QString& filePath, const RawDecodingSettings& rawDecodingSettings, 
-                            QByteArray &imageData, int &width, int &height, int &rgbmax)
+                            QByteArray& imageData, int& width, int& height, int& rgbmax)
 {
     m_rawDecodingSettings = rawDecodingSettings;
-    return (loadFromDcraw(filePath, imageData, width, height, rgbmax));
+    return (loadFromLibraw(filePath, imageData, width, height, rgbmax));
 }
 
 bool KDcraw::checkToCancelWaitingData()
@@ -351,21 +350,12 @@ bool KDcraw::checkToCancelWaitingData()
     return m_cancel;
 }
 
-bool KDcraw::checkToCancelReceivingData()
-{
-    return m_cancel;
-}
-
 void KDcraw::setWaitingDataProgress(double)
 {
 }
 
-void KDcraw::setReceivingDataProgress(double)
-{
-}
-
-bool KDcraw::loadFromDcraw(const QString& filePath, QByteArray &imageData, 
-                           int &width, int &height, int &rgbmax)
+bool KDcraw::loadFromLibraw(const QString& filePath, QByteArray& imageData,
+                            int& width, int& height, int& rgbmax)
 {
     m_cancel = false;
 
@@ -389,12 +379,6 @@ bool KDcraw::loadFromDcraw(const QString& filePath, QByteArray &imageData,
         raw.imgdata.params.no_auto_bright = 1;
     }
 
-    if (m_rawDecodingSettings.gamma16bit)
-    {
-        // 16 bits color depth auto-gamma is not implemented in dcraw.
-        raw.imgdata.params.gamma_16bit = 1;
-    }
-
     if (m_rawDecodingSettings.sixteenBitsImage)
     {
         // (-4) 16bit ppm output
