@@ -43,7 +43,7 @@ bool KExiv2::canWriteIptc(const QString& filePath)
     catch( Exiv2::Error &e )
     {
         std::string s(e.what());
-        kDebug(51003) << "Cannot check Iptc access mode using Exiv2 (Error #" 
+        kDebug(51003) << "Cannot check Iptc access mode using Exiv2 (Error #"
                       << e.code() << ": " << s.c_str() << ")" << endl;
     }
 
@@ -52,14 +52,14 @@ bool KExiv2::canWriteIptc(const QString& filePath)
 
 bool KExiv2::hasIptc() const
 {
-    return !d->iptcMetadata.empty();
+    return !d->iptcMetadata().empty();
 }
 
 bool KExiv2::clearIptc() const
 {
     try
     {
-        d->iptcMetadata.clear();
+        d->iptcMetadata().clear();
         return true;
     }
     catch( Exiv2::Error &e )
@@ -74,12 +74,12 @@ QByteArray KExiv2::getIptc(bool addIrbHeader) const
 {
     try
     {
-        if (!d->iptcMetadata.empty())
+        if (!d->iptcMetadata().empty())
         {
-            Exiv2::IptcData& iptc = d->iptcMetadata;
+            Exiv2::IptcData& iptc = d->iptcMetadata();
             Exiv2::DataBuf c2;
 
-            if (addIrbHeader) 
+            if (addIrbHeader)
             {
 #if (EXIV2_TEST_VERSION(0,10,0))
                 c2 = Exiv2::Photoshop::setIptcIrb(0, 0, iptc);
@@ -88,10 +88,10 @@ QByteArray KExiv2::getIptc(bool addIrbHeader) const
                 return QByteArray();
 #endif
             }
-            else 
+            else
             {
 #if (EXIV2_TEST_VERSION(0,17,91))
-                c2 = Exiv2::IptcParser::encode(d->iptcMetadata);
+                c2 = Exiv2::IptcParser::encode(d->iptcMetadata());
 #else
                 c2 = iptc.copy();
 #endif
@@ -120,10 +120,10 @@ bool KExiv2::setIptc(const QByteArray& data) const
         if (!data.isEmpty())
         {
 #if (EXIV2_TEST_VERSION(0,17,91))
-            Exiv2::IptcParser::decode(d->iptcMetadata, (const Exiv2::byte*)data.data(), data.size());
-            return (!d->iptcMetadata.empty());
+            Exiv2::IptcParser::decode(d->iptcMetadata(), (const Exiv2::byte*)data.data(), data.size());
+            return (!d->iptcMetadata().empty());
 #else
-            if (d->iptcMetadata.load((const Exiv2::byte*)data.data(), data.size()) != 0)
+            if (d->iptcMetadata().load((const Exiv2::byte*)data.data(), data.size()) != 0)
                 return false;
             else
                 return true;
@@ -143,12 +143,12 @@ bool KExiv2::setIptc(const QByteArray& data) const
 
 KExiv2::MetaDataMap KExiv2::getIptcTagsDataList(const QStringList &iptcKeysFilter, bool invertSelection) const
 {
-    if (d->iptcMetadata.empty())
+    if (d->iptcMetadata().empty())
        return MetaDataMap();
 
     try
     {
-        Exiv2::IptcData iptcData = d->iptcMetadata;
+        Exiv2::IptcData iptcData = d->iptcMetadata();
         iptcData.sortByKey();
 
         QString     ifDItemName;
@@ -214,13 +214,13 @@ KExiv2::MetaDataMap KExiv2::getIptcTagsDataList(const QStringList &iptcKeysFilte
 
 QString KExiv2::getIptcTagTitle(const char *iptcTagName)
 {
-    try 
+    try
     {
         std::string iptckey(iptcTagName);
-        Exiv2::IptcKey ik(iptckey); 
+        Exiv2::IptcKey ik(iptckey);
         return QString::fromLocal8Bit( Exiv2::IptcDataSets::dataSetTitle(ik.tag(), ik.record()) );
     }
-    catch (Exiv2::Error& e) 
+    catch (Exiv2::Error& e)
     {
         d->printExiv2ExceptionError("Cannot get metadata tag title using Exiv2 ", e);
     }
@@ -233,10 +233,10 @@ QString KExiv2::getIptcTagDescription(const char *iptcTagName)
     try
     {
         std::string iptckey(iptcTagName);
-        Exiv2::IptcKey ik(iptckey); 
+        Exiv2::IptcKey ik(iptckey);
         return QString::fromLocal8Bit( Exiv2::IptcDataSets::dataSetDesc(ik.tag(), ik.record()) );
     }
-    catch (Exiv2::Error& e) 
+    catch (Exiv2::Error& e)
     {
         d->printExiv2ExceptionError("Cannot get metadata tag description using Exiv2 ", e);
     }
@@ -251,15 +251,15 @@ bool KExiv2::removeIptcTag(const char *iptcTagName, bool setProgramName) const
 
     try
     {
-        Exiv2::IptcData::iterator it = d->iptcMetadata.begin();
+        Exiv2::IptcData::iterator it = d->iptcMetadata().begin();
         int i = 0;
-        while(it != d->iptcMetadata.end())
+        while(it != d->iptcMetadata().end())
         {
             QString key = QString::fromLocal8Bit(it->key().c_str());
 
             if (key == QString(iptcTagName))
             {
-                it = d->iptcMetadata.erase(it);
+                it = d->iptcMetadata().erase(it);
                 ++i;
             }
             else
@@ -290,7 +290,7 @@ bool KExiv2::setIptcTagData(const char *iptcTagName, const QByteArray& data, boo
     try
     {
         Exiv2::DataValue val((Exiv2::byte *)data.data(), data.size());
-        d->iptcMetadata[iptcTagName] = val;
+        d->iptcMetadata()[iptcTagName] = val;
         return true;
     }
     catch( Exiv2::Error &e )
@@ -306,14 +306,14 @@ QByteArray KExiv2::getIptcTagData(const char *iptcTagName) const
     try
     {
         Exiv2::IptcKey iptcKey(iptcTagName);
-        Exiv2::IptcData iptcData(d->iptcMetadata);
+        Exiv2::IptcData iptcData(d->iptcMetadata());
         Exiv2::IptcData::iterator it = iptcData.findKey(iptcKey);
         if (it != iptcData.end())
         {
             char *s = new char[(*it).size()];
             (*it).copy((Exiv2::byte*)s, Exiv2::bigEndian);
             QByteArray data(s, (*it).size());
-            delete s;
+            delete [] s;
             return data;
         }
     }
@@ -331,7 +331,7 @@ QString KExiv2::getIptcTagString(const char* iptcTagName, bool escapeCR) const
     try
     {
         Exiv2::IptcKey iptcKey(iptcTagName);
-        Exiv2::IptcData iptcData(d->iptcMetadata);
+        Exiv2::IptcData iptcData(d->iptcMetadata());
         Exiv2::IptcData::iterator it = iptcData.findKey(iptcKey);
         if (it != iptcData.end())
         {
@@ -361,7 +361,7 @@ bool KExiv2::setIptcTagString(const char *iptcTagName, const QString& value, boo
 
     try
     {
-        d->iptcMetadata[iptcTagName] = std::string(value.toAscii().constData());
+        d->iptcMetadata()[iptcTagName] = std::string(value.toAscii().constData());
         return true;
     }
     catch( Exiv2::Error &e )
@@ -376,10 +376,10 @@ QStringList KExiv2::getIptcTagsStringList(const char* iptcTagName, bool escapeCR
 {
     try
     {
-        if (!d->iptcMetadata.empty())
+        if (!d->iptcMetadata().empty())
         {
             QStringList values;
-            Exiv2::IptcData iptcData(d->iptcMetadata);
+            Exiv2::IptcData iptcData(d->iptcMetadata());
 
             for (Exiv2::IptcData::iterator it = iptcData.begin(); it != iptcData.end(); ++it)
             {
@@ -409,7 +409,7 @@ QStringList KExiv2::getIptcTagsStringList(const char* iptcTagName, bool escapeCR
 }
 
 bool KExiv2::setIptcTagsStringList(const char* iptcTagName, int maxSize,
-                                   const QStringList& oldValues, const QStringList& newValues, 
+                                   const QStringList& oldValues, const QStringList& newValues,
                                    bool setProgramName) const
 {
     if (!setProgramId(setProgramName))
@@ -420,11 +420,11 @@ bool KExiv2::setIptcTagsStringList(const char* iptcTagName, int maxSize,
         QStringList oldvals = oldValues;
         QStringList newvals = newValues;
 
-        kDebug(51003) << d->filePath.toAscii().constData() << " : " << iptcTagName 
+        kDebug(51003) << d->filePath.toAscii().constData() << " : " << iptcTagName
                       << " => " << newvals.join(",").toAscii().constData() << endl;
 
         // Remove all old values.
-        Exiv2::IptcData iptcData(d->iptcMetadata);
+        Exiv2::IptcData iptcData(d->iptcMetadata());
         Exiv2::IptcData::iterator it = iptcData.begin();
 
         while(it != iptcData.end())
@@ -437,7 +437,7 @@ bool KExiv2::setIptcTagsStringList(const char* iptcTagName, int maxSize,
                  (oldvals.contains(val) || newvals.contains(val))
                )
                 it = iptcData.erase(it);
-            else 
+            else
                 ++it;
         };
 
@@ -455,7 +455,7 @@ bool KExiv2::setIptcTagsStringList(const char* iptcTagName, int maxSize,
             iptcData.add(iptcTag, val.get());
         }
 
-        d->iptcMetadata = iptcData;
+        d->iptcMetadata() = iptcData;
 
         return true;
     }
@@ -472,10 +472,10 @@ QStringList KExiv2::getIptcKeywords() const
 {
     try
     {
-        if (!d->iptcMetadata.empty())
+        if (!d->iptcMetadata().empty())
         {
             QStringList keywords;
-            Exiv2::IptcData iptcData(d->iptcMetadata);
+            Exiv2::IptcData iptcData(d->iptcMetadata());
 
             for (Exiv2::IptcData::iterator it = iptcData.begin(); it != iptcData.end(); ++it)
             {
@@ -499,7 +499,7 @@ QStringList KExiv2::getIptcKeywords() const
     return QStringList();
 }
 
-bool KExiv2::setIptcKeywords(const QStringList& oldKeywords, const QStringList& newKeywords, 
+bool KExiv2::setIptcKeywords(const QStringList& oldKeywords, const QStringList& newKeywords,
                              bool setProgramName) const
 {
     if (!setProgramId(setProgramName))
@@ -510,11 +510,11 @@ bool KExiv2::setIptcKeywords(const QStringList& oldKeywords, const QStringList&
         QStringList oldkeys = oldKeywords;
         QStringList newkeys = newKeywords;
 
-        kDebug(51003) << d->filePath.toAscii().constData() 
+        kDebug(51003) << d->filePath.toAscii().constData()
                       << " ==> Iptc Keywords: " << newkeys.join(",").toAscii().constData() << endl;
 
         // Remove all old keywords.
-        Exiv2::IptcData iptcData(d->iptcMetadata);
+        Exiv2::IptcData iptcData(d->iptcMetadata());
         Exiv2::IptcData::iterator it = iptcData.begin();
 
         while(it != iptcData.end())
@@ -526,7 +526,7 @@ bool KExiv2::setIptcKeywords(const QStringList& oldKeywords, const QStringList&
             if ( key == QString("Iptc.Application2.Keywords") &&
                  (oldKeywords.contains(val) || newKeywords.contains(val)) )
                 it = iptcData.erase(it);
-            else 
+            else
                 ++it;
         };
 
@@ -544,7 +544,7 @@ bool KExiv2::setIptcKeywords(const QStringList& oldKeywords, const QStringList&
             iptcData.add(iptcTag, val.get());
         }
 
-        d->iptcMetadata = iptcData;
+        d->iptcMetadata() = iptcData;
 
         return true;
     }
@@ -560,10 +560,10 @@ QStringList KExiv2::getIptcSubjects() const
 {
     try
     {
-        if (!d->iptcMetadata.empty())
+        if (!d->iptcMetadata().empty())
         {
             QStringList subjects;
-            Exiv2::IptcData iptcData(d->iptcMetadata);
+            Exiv2::IptcData iptcData(d->iptcMetadata());
 
             for (Exiv2::IptcData::iterator it = iptcData.begin(); it != iptcData.end(); ++it)
             {
@@ -587,7 +587,7 @@ QStringList KExiv2::getIptcSubjects() const
     return QStringList();
 }
 
-bool KExiv2::setIptcSubjects(const QStringList& oldSubjects, const QStringList& newSubjects, 
+bool KExiv2::setIptcSubjects(const QStringList& oldSubjects, const QStringList& newSubjects,
                              bool setProgramName) const
 {
     if (!setProgramId(setProgramName))
@@ -599,7 +599,7 @@ bool KExiv2::setIptcSubjects(const QStringList& oldSubjects, const QStringList&
         QStringList newDef = newSubjects;
 
         // Remove all old subjects.
-        Exiv2::IptcData iptcData(d->iptcMetadata);
+        Exiv2::IptcData iptcData(d->iptcMetadata());
         Exiv2::IptcData::iterator it = iptcData.begin();
 
         while(it != iptcData.end())
@@ -609,7 +609,7 @@ bool KExiv2::setIptcSubjects(const QStringList& oldSubjects, const QStringList&
 
             if (key == QString("Iptc.Application2.Subject") && oldDef.contains(val))
                 it = iptcData.erase(it);
-            else 
+            else
                 ++it;
         };
 
@@ -627,7 +627,7 @@ bool KExiv2::setIptcSubjects(const QStringList& oldSubjects, const QStringList&
             iptcData.add(iptcTag, val.get());
         }
 
-        d->iptcMetadata = iptcData;
+        d->iptcMetadata() = iptcData;
 
         return true;
     }
@@ -643,10 +643,10 @@ QStringList KExiv2::getIptcSubCategories() const
 {
     try
     {
-        if (!d->iptcMetadata.empty())
+        if (!d->iptcMetadata().empty())
         {
             QStringList subCategories;
-            Exiv2::IptcData iptcData(d->iptcMetadata);
+            Exiv2::IptcData iptcData(d->iptcMetadata());
 
             for (Exiv2::IptcData::iterator it = iptcData.begin(); it != iptcData.end(); ++it)
             {
@@ -670,7 +670,7 @@ QStringList KExiv2::getIptcSubCategories() const
     return QStringList();
 }
 
-bool KExiv2::setIptcSubCategories(const QStringList& oldSubCategories, const QStringList& newSubCategories, 
+bool KExiv2::setIptcSubCategories(const QStringList& oldSubCategories, const QStringList& newSubCategories,
                                   bool setProgramName) const
 {
     if (!setProgramId(setProgramName))
@@ -682,7 +682,7 @@ bool KExiv2::setIptcSubCategories(const QStringList& oldSubCategories, const QSt
         QStringList newkeys = newSubCategories;
 
         // Remove all old Sub Categories.
-        Exiv2::IptcData iptcData(d->iptcMetadata);
+        Exiv2::IptcData iptcData(d->iptcMetadata());
         Exiv2::IptcData::iterator it = iptcData.begin();
 
         while(it != iptcData.end())
@@ -692,11 +692,11 @@ bool KExiv2::setIptcSubCategories(const QStringList& oldSubCategories, const QSt
 
             if (key == QString("Iptc.Application2.SuppCategory") && oldSubCategories.contains(val))
                 it = iptcData.erase(it);
-            else 
+            else
                 ++it;
         };
 
-        // Add new Sub Categories. Note that SubCategories Iptc tag is limited to 32 
+        // Add new Sub Categories. Note that SubCategories Iptc tag is limited to 32
         // characters but can be redondant.
 
         Exiv2::IptcKey iptcTag("Iptc.Application2.SuppCategory");
@@ -711,7 +711,7 @@ bool KExiv2::setIptcSubCategories(const QStringList& oldSubCategories, const QSt
             iptcData.add(iptcTag, val.get());
         }
 
-        d->iptcMetadata = iptcData;
+        d->iptcMetadata() = iptcData;
 
         return true;
     }
@@ -723,4 +723,35 @@ bool KExiv2::setIptcSubCategories(const QStringList& oldSubCategories, const QSt
     return false;
 }
 
+KExiv2::TagsMap KExiv2::getIptcTagsList() const
+{
+    try
+    {
+        QList<const Exiv2::DataSet*> tags;
+        tags << Exiv2::IptcDataSets::envelopeRecordList()
+             << Exiv2::IptcDataSets::application2RecordList();
+
+        TagsMap tagsMap;
+        for (QList<const Exiv2::DataSet*>::iterator it = tags.begin(); it != tags.end(); ++it)
+        {
+            do
+            {
+                QString     key = QLatin1String( Exiv2::IptcKey( (*it)->number_, (*it)->recordId_ ).key().c_str() );
+                QStringList values;
+                values << (*it)->name_ << (*it)->title_ << (*it)->desc_;
+                tagsMap.insert(key, values);
+                ++(*it);
+            }
+            while((*it)->number_ != 0xffff);
+        }
+        return tagsMap;
+    }
+    catch( Exiv2::Error &e )
+    {
+        d->printExiv2ExceptionError("Cannot get Iptc Tags list using Exiv2 ", e);
+    }
+
+    return TagsMap();
+}
+
 }  // NameSpace KExiv2Iface
