@@ -9,6 +9,7 @@
  * Copyright (C) 2004-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
  * Copyright (C) 2009 by Andi Clemens <andi dot clemens at gmx dot net>
  * Copyright (C) 2004-2005 by Renchi Raju <renchi.raju at kdemail.net>
+ * Copyright (C) 2009 by Aleix Pol Gonzalez <aleixpol at kde dot org>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -24,7 +25,6 @@
 
 /** @file pluginloader.cpp */
 
-#include "pluginloader.h"
 #include "pluginloader.moc"
 
 // Qt include.
@@ -43,6 +43,7 @@
 #include <kconfig.h>
 #include <kglobal.h>
 #include <klibloader.h>
+#include <kaction.h>
 
 // Local includes.
 
@@ -140,6 +141,7 @@ PluginLoader::Info::Info(const KService::Ptr& service, bool shouldLoad)
 
 PluginLoader::Info::~Info()
 {
+    delete d->m_plugin;
     delete d;
 }
 
@@ -163,13 +165,48 @@ QString PluginLoader::Info::library() const
     return d->m_service->library();
 }
 
+QIcon PluginLoader::Info::icon() const
+{
+    if (d->m_plugin) return QIcon(d->m_plugin->actions()[0]->icon());
+    else return KIcon(d->m_service->icon());
+}
+
 Plugin* PluginLoader::Info::plugin() const
 {
+    if ( !d->m_plugin && shouldLoad() )
+    {
+        QString error;
+        Plugin *plugin = d->m_service->createInstance<Plugin>(PluginLoader::instance()->interface(), QVariantList(), &error);
+        if (plugin)
+        {
+            kDebug( 51001 ) << "KIPI::PluginLoader: Loaded plugin " << plugin->objectName() << endl;
+        }
+        else
+        {
+            kWarning( 51001 ) << "KIPI::PluginLoader:: createInstance returned 0 for "
+                              << name()
+                              << " (" << library() << ")"
+                              << " with error: "
+                              << error;
+        }
+        d->m_plugin=plugin;
+        
+        if ( d->m_plugin ) // Do not emit if we had trouble loading the plugin.
+            emit PluginLoader::instance()->plug( const_cast<Info*>(this) );
+    }
+    
     return d->m_plugin;
 }
 
+void PluginLoader::Info::reload()
+{
+    delete d->m_plugin;
+    d->m_plugin=0;
+}
+
 void PluginLoader::Info::setPlugin(Plugin* plugin)
 {
+    delete d->m_plugin;
     d->m_plugin=plugin;
 }
 
@@ -196,19 +233,28 @@ public:
         m_interface = 0;
     };
 
-    QStringList               m_ignores;
     PluginLoader::PluginList  m_pluginList;
     Interface*                m_interface;
 };
 
-PluginLoader::PluginLoader( const QStringList& ignores, Interface* interface )
+PluginLoader::PluginLoader( const QStringList& ignores, Interface* interface)
             : d(new PluginLoaderPrivate)
+{
+    construct(ignores, interface, QString());
+}
+
+PluginLoader::PluginLoader( const QStringList& ignores, Interface* interface, const QString& constraint )
+            : d(new PluginLoaderPrivate)
+{
+    construct(ignores, interface, constraint);
+}
+
+void PluginLoader::construct( const QStringList& ignores, Interface* interface, const QString& constraint )
 {
     Q_ASSERT( s_instance == 0 );
     s_instance                  = this;
     d->m_interface              = interface;
-    d->m_ignores                = ignores;
-    const KService::List offers = KServiceTypeTrader::self()->query("KIPI/Plugin");
+    const KService::List offers = KServiceTypeTrader::self()->query("KIPI/Plugin", constraint);
     KSharedConfigPtr config     = KGlobal::config();
     KConfigGroup group          = config->group( QString::fromLatin1( "KIPI/EnabledPlugin" ) );
 
@@ -217,7 +263,6 @@ PluginLoader::PluginLoader( const QStringList& ignores, Interface* interface )
     {
         KService::Ptr service   = *iter;
         QString name            = service->name();
-        QString comment         = service->comment();
         QString library         = service->library();
         QStringList reqFeatures = service->property( QString::fromLatin1( "X-KIPI-ReqFeatures" ) ).toStringList();
 
@@ -227,7 +272,7 @@ PluginLoader::PluginLoader( const QStringList& ignores, Interface* interface )
             continue;
         }
 
-        if ( d->m_ignores.contains( name ) )
+        if ( ignores.contains( name ) )
         {
             kDebug( 51001 ) << "KIPI::PluginLoader: plugin " << name 
                             << " is in the ignore list for host application" << endl;
@@ -259,41 +304,19 @@ PluginLoader::PluginLoader( const QStringList& ignores, Interface* interface )
 
 PluginLoader::~PluginLoader()
 {
+    qDeleteAll(d->m_pluginList);
     delete d;
 }
 
+///deprecated
 void PluginLoader::loadPlugins()
 {
-    for( PluginList::Iterator it = d->m_pluginList.begin(); it != d->m_pluginList.end(); ++it )
-    {
-        loadPlugin( *it );
-    }
-    emit replug();
+    emit replug(); //added for convenience, now they will be loaded on demand
 }
 
-void PluginLoader::loadPlugin( Info* info )
-{
-    if ( info->plugin() == 0 && info->shouldLoad() )
-    {
-        QString error;
-        Plugin *plugin = info->service()->createInstance<Plugin>(d->m_interface, QVariantList(), &error);
-        if (plugin)
-        {
-            kDebug( 51001 ) << "KIPI::PluginLoader: Loaded plugin " << plugin->objectName() << endl;
-        }
-        else
-        {
-            kWarning( 51001 ) << "KIPI::PluginLoader:: createInstance returned 0 for "
-                              << info->name()
-                              << " (" << info->library() << ")"
-                              << " with error: "
-                              << error << endl;
-        }
-        info->setPlugin(plugin);
-    }
-    if ( info->plugin() ) // Do not emit if we had trouble loading the plugin.
-        emit PluginLoader::instance()->plug( info );
-}
+///deprecated
+void PluginLoader::loadPlugin( Info* )
+{}
 
 const PluginLoader::PluginList& PluginLoader::pluginList()
 {
@@ -308,6 +331,12 @@ PluginLoader* PluginLoader::instance()
     return s_instance;
 }
 
+Interface* PluginLoader::interface() const
+{
+    return d->m_interface;
+}
+
+
 //---------------------------------------------------------------------
 
 ConfigWidget* PluginLoader::configWidget(QWidget* parent)
@@ -323,6 +352,7 @@ public:
                                : QListWidgetItem(parent, QListWidgetItem::UserType), info(info)
     {
         setText(QString("%1  (%2)").arg(info->name(), info->comment()));
+        setIcon(info->icon());
         setFlags(Qt::ItemIsUserCheckable|Qt::ItemIsEnabled);
         setCheckState(info->shouldLoad() ? Qt::Checked : Qt::Unchecked);
     }
@@ -374,13 +404,10 @@ void ConfigWidget::apply()
             (*it)->info->setShouldLoad(load);
             if ( load )
             {
-                PluginLoader::instance()->loadPlugin( (*it)->info);
-            }
-            else
-            {
-                if ( (*it)->info->plugin() ) // Do not emit if we had trouble loading plugin.
-                    emit PluginLoader::instance()->unplug( (*it)->info);
+                (*it)->info->reload();
             }
+            else if ( (*it)->info->plugin() ) // Do not emit if we had trouble loading plugin.
+                emit PluginLoader::instance()->unplug( (*it)->info); 
         }
     }
     emit PluginLoader::instance()->replug();
