@@ -30,7 +30,9 @@
 
 // Qt includes.
 #include <QFrame>
-#include <QString>
+
+//KDE includes
+#include <KLocalizedString>
 
 // Sane includes.
 extern "C"
@@ -79,6 +81,7 @@ public:
     virtual bool setValue(float val);
     virtual bool getValue(QString &val);
     virtual bool setValue(const QString &val);
+    virtual int  getUnit();
 
     bool storeCurrentData();
     bool restoreSavedData();
@@ -101,7 +104,8 @@ protected:
     SANE_Word toSANE_Word(unsigned char *data);
     void fromSANE_Word(unsigned char *data, SANE_Word from);
     bool writeData(void *data);
-    QString unitString();
+    KLocalizedString unitString();
+    QString unitDoubleString();
     KSaneOptWState state();
     void updateVisibility();
     
