@@ -36,7 +36,7 @@ static KAboutData createAboutData()
          "okular_chm",
          "okular_chm",
          ki18n( "CHM Backend" ),
-         "0.1.3",
+         "0.1.4",
          ki18n( "A Microsoft Windows help file renderer" ),
          KAboutData::License_GPL,
          ki18n( "© 2005-2007 Piotr Szymański\n© 2008 Albert Astals Cid" )
@@ -84,9 +84,14 @@ CHMGenerator::~CHMGenerator()
 
 bool CHMGenerator::loadDocument( const QString & fileName, QVector< Okular::Page * > & pagesVector )
 {
+    m_file = new LCHMFile();
+    if (!m_file->loadFile(fileName))
+    {
+        delete m_file;
+        m_file = 0;
+        return false;
+    }
     m_fileName=fileName;
-    m_file=new LCHMFile ();
-    m_file->loadFile(fileName);
     QVector< LCHMParsedEntry > topics;
     m_file->parseTableOfContents(&topics);
     
