@@ -19,7 +19,7 @@
 #ifndef _OKULAR_PAGEVIEW_H_
 #define _OKULAR_PAGEVIEW_H_
 
-#include <qscrollarea.h>
+#include <qabstractscrollarea.h>
 #include <qlist.h>
 #include <qvector.h>
 #include "ui/pageviewutils.h"
@@ -39,19 +39,16 @@ class Annotation;
 
 class FormWidgetIface;
 class PageViewPrivate;
-class PageViewWidget;
 
 /**
  * @short The main view. Handles zoom and continuous mode.. oh, and page
  * @short display of course :-)
  * ...
  */
-class PageView : public QScrollArea, public Okular::DocumentObserver, public Okular::View
+class PageView : public QAbstractScrollArea, public Okular::DocumentObserver, public Okular::View
 {
 Q_OBJECT
 
-    friend class PageViewWidget;
-
     public:
         PageView( QWidget *parent, Okular::Document *document );
         ~PageView();
@@ -94,6 +91,11 @@ Q_OBJECT
 
         KAction *toggleFormsAction() const;
 
+        int contentAreaWidth() const;
+        int contentAreaHeight() const;
+        QPoint contentAreaPosition() const;
+        QPoint contentAreaPoint( const QPoint & pos ) const;
+
     public slots:
         void errorMessage( const QString & message, int duration = -1 )
         {
@@ -135,12 +137,16 @@ Q_OBJECT
         void dragMoveEvent( QDragMoveEvent* );
         void dropEvent( QDropEvent* );
 
-    private:
-        void contentsPaintEvent( QPaintEvent *e );
-        void contentsMouseMoveEvent( QMouseEvent *e );
-        void contentsMousePressEvent( QMouseEvent *e );
-        void contentsMouseReleaseEvent( QMouseEvent *e );
+        void paintEvent( QPaintEvent *e );
+        void mouseMoveEvent( QMouseEvent *e );
+        void mousePressEvent( QMouseEvent *e );
+        void mouseReleaseEvent( QMouseEvent *e );
 
+        bool viewportEvent( QEvent *e );
+
+        void scrollContentsBy( int dx, int dy );
+
+    private:
         // draw background and items on the opened qpainter
         void drawDocumentOnPainter( const QRect & pageViewRect, QPainter * p );
         // update item width and height using current zoom parameters
@@ -166,6 +172,8 @@ Q_OBJECT
 
         void toggleFormWidgets( bool on );
 
+        void resizeContentArea( const QSize & newSize );
+
         // don't want to expose classes in here
         class PageViewPrivate * d;
 
